@@ -1,15 +1,16 @@
-/* readinputfile.h -- This file is part of Archimedes release 0.0.5.
-   Archimedes is a simulator for Submicron 2D Silicon/GaAs
+/* readinputfile.h -- This file is part of Archimedes release 0.1.0
+   Archimedes is a simulator for Submicron 2D III-V semiconductor
    Devices. It implements the Monte Carlo method and Hybrid MEP model
    for the simulation of the semiclassical Boltzmann equation for both
    electrons and holes. It also includes the quantum effects by means 
-   of effective potential method.
+   of effective potential method. It is now able to simulate applied
+   magnetic fields along with self consistent Faraday equation.
 
-   Copyright (C) 2004, 2005 Jean Michel Sellier <sellier@dmi.unict.it>
+   Copyright (C) 2004, 2005, 2006, 2007 Jean Michel Sellier <sellier@dmi.unict.it>
  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -18,26 +19,25 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-   USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
 
 
 // ######################################################
 // Created on 10 sep.2004, Siracusa, J.M.Sellier
-// Last modif. : 07 apr.2005, Siracusa, J.M.Sellier
+// Last modif. : 10 september 2007, Catania, J.M.Sellier
 // ######################################################
 
 // Read the geometrical and physical description of the device.
 // The input file is interpreted as in a script language.
 // For the syntax of the commands of this language see
-// the manual GNU Archimedes release 0.0.5
+// the manual GNU Archimedes release 0.0.7
 
 void
 Read_Input_File(void)
 {
- char s[80];
- long double num,num0;
+ char s[180];
+ double num,num0;
  int ini,fin;
  int i,j;
  int LXflag=0, LYflag=0;
@@ -49,14 +49,13 @@ Read_Input_File(void)
 // the following values.
 // ============================
  Model_Number=MCE; // here we choose the model
- Material=SILICON; // MESFET made of Silicon
  nx=50; // default # of cells in x-direction
  ny=50; // default # of cells in y-direction
  TF=5.0e-12; // Default Final Time in seconds
  DT=0.001e-12; // Default constant time step
  TAUW=0.4e-12; // Default Energy relaxation time
- mstar=mstarSILICON; // coefficient of the Si elettronic effective mass
  msh=mstarHSILICON; // coefficient of the Si heavy hole effective mass
+ FARADAYFLAG=0; // no evolution of magnetic field taken into account as default (B = 0 or const.)
 // standard doping concentration
  for(i=1;i<=nx+1;i++)
    for(j=1;j<=ny+1;j++){
@@ -64,6 +63,8 @@ Read_Input_File(void)
       h2d[i][j][1]=N_H[i][j]=NI;
       u2d[i][j][2]=u2d[i][j][3]=0.;
       h2d[i][j][2]=h2d[i][j][3]=0.;
+      i_dom[i][j]=SILICON; // Silicon everywhere as default
+      B[i][j]=0.; // no magnetic field as default
    }
  CIMP=1.e20; // Impurity concentration
  TL=300.; // Lattice temperature in Kelvin
@@ -93,29 +94,46 @@ Processing the input file\n\
   }
 // read and check if the material specified exists
   else if(strcmp(s,"MATERIAL")==0){
+    int type;
+    real xi,xf;
+    real yi,yf;
+    fscanf(fp,"%s",s);
+    if(strcmp(s,"X")==0) fscanf(fp,"%lf %lf",&xi,&xf);
+    else{
+      printf("%s : X material not specified!\n",progname);
+      exit(0);
+    }
+    fscanf(fp,"%s",s);
+    if(strcmp(s,"Y")==0) fscanf(fp,"%lf %lf",&yi,&yf);
+    else{
+      printf("%s : Y material not specified!\n",progname);
+      exit(0);
+    }
     fscanf(fp,"%s",s);
     if(strcmp(s,"SILICON")==0){
-      Material=SILICON;
-      epsMATERIAL=EPSR*EPS0;
-      alphaMATERIAL=alphaSILICON;
-      mstar=mstarSILICON;
-      msh=mstarHSILICON;
-      printf("MATERIAL %s ---> Ok\n",s);
+        type=SILICON;
+        msh=mstarHSILICON;
     }
     else if(strcmp(s,"GAAS")==0){
-      Material=GAAS;
-      epsMATERIAL=EPS_GAAS*EPS0;
-      alphaMATERIAL=alphaGAAS;
-      mstar=mstarGAAS;
-      mstar2=mstar2GAAS;
-      msh=mstarHGAAS;
-      printf("MATERIAL %s ---> Ok\n",s);
+        type=GAAS;
+        msh=mstarHGAAS;
+    }
+    else if(strcmp(s,"GERMANIUM")==0){
+        type=GERMANIUM;
+        msh=mstarHGERMANIUM;
     }
-// if the material specified does not exist then exit
     else{
-      printf("%s: unknow specified material\n",progname);
-      exit(EXIT_FAILURE);
+      printf("%s : unknown specified material!\n",progname);
+      exit(0);
     }
+    for(i=0;i<=nx+4;i++)
+      for(j=0;j<=ny+4;j++){
+        if((i-0.5)*dx>=xi && (i-1.5)*dx<=xf
+         &&(j-0.5)*dy>=yi && (j-1.5)*dy<=yf){
+           i_dom[i][j]=type;
+        }
+      }
+    printf("MATERIAL %s X=[%g,%g] Y=[%g,%g] ---> Ok\n",s,xi,xf,yi,yf);
   }
 // here we choose the charge particle transport simulated
   else if(strcmp(s,"TRANSPORT")==0){
@@ -170,7 +188,7 @@ Processing the input file\n\
   }
 // Specified the number of spatial step in x direction
   else if(strcmp(s,"XSPATIALSTEP")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     nx=(int) num;
     if(nx>NXM){
       printf("%s: too large x-spatial step\n",progname);
@@ -185,7 +203,7 @@ Processing the input file\n\
   }
 // Specified the number of spatial step in y direction
   else if(strcmp(s,"YSPATIALSTEP")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     ny=(int) num;
     if(ny>NYM){
       printf("%s: too large y-spatial step\n",progname);
@@ -200,63 +218,62 @@ Processing the input file\n\
   }
 // configuration of the final time of simulation
   else if(strcmp(s,"FINALTIME")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lg",&num);
     TF=num;
     if(TF<=0.){
       printf("%s: not valid final time\n",progname);
-      exit(EXIT_FAILURE);
     }
-    printf("FINAL TIME = %g ---> Ok\n",TF);
+    printf("FINAL TIME = %lg ---> Ok\n",TF);
   }
 // Setting the Silicon Oxyde interface
   else if(strcmp(s,"OXYDE")==0){
     fscanf(fp,"%s",s);
     if(strcmp(s,"UP")==0){
       SIO2_UP_FLAG=1;
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       SIO2_INI[0]=num;
       if(num<0.){
         printf("%s: not valid upper SiO2 initial position\n",progname);
         exit(EXIT_FAILURE);
       }
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       SIO2_FIN[0]=num;
       if(num<=SIO2_INI[0] || num>LX){
         printf("%s: not valid upper SiO2 final position\n",progname);
         exit(EXIT_FAILURE);
       }
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       if(num<=0.){
         printf("%s: not valid upper SiO2 thickness\n",progname);
         exit(EXIT_FAILURE);
       }
       SIO2_THICKNESS[0]=num;
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       SIO2_POT[0]=num;
       printf("OXYDE ---> UP %g %g %g %g\n",
               SIO2_INI[0],SIO2_FIN[0],SIO2_THICKNESS[0],SIO2_POT[0]);
     }
     else if(strcmp(s,"DOWN")==0){
       SIO2_DOWN_FLAG=1;
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       SIO2_INI[1]=num;
       if(num<0.){
         printf("%s: not valid lower SiO2 initial position\n",progname);
         exit(EXIT_FAILURE);
       }
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       SIO2_FIN[1]=num;
       if(num<=SIO2_INI[1] || num>LX){
         printf("%s: not valid lower SiO2 final position\n",progname);
         exit(EXIT_FAILURE);
       }
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       if(num<=0.){
         printf("%s: not valid lower SiO2 thickness\n",progname);
         exit(EXIT_FAILURE);
       }
       SIO2_THICKNESS[1]=num;
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       SIO2_POT[1]=num;
       printf("OXYDE ---> DOWN %g %g %g %g\n",
               SIO2_INI[1],SIO2_FIN[1],SIO2_THICKNESS[1],SIO2_POT[1]);
@@ -269,7 +286,7 @@ Processing the input file\n\
   }
 // setting the energy relaxation time
   else if(strcmp(s,"TAUW")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     TAUW=num;
     if(TAUW==0.){
       printf("%s: not valid energy relaxation time\n",progname);
@@ -279,7 +296,7 @@ Processing the input file\n\
   }
 // configuration of the time step
   else if(strcmp(s,"TIMESTEP")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     DT=num;
     if(DT<=0.){
       printf("%s: not valid time step\n",progname);
@@ -292,7 +309,7 @@ Processing the input file\n\
     FILE *dp;
     FILE *pp;
     FILE *ep;
-    long double dum,dum0,dum1;
+    double dum,dum0,dum1;
 // Open the initial electron density file
     dp=fopen("density_start.xyz","r");
 // File Control, just in case the file does not exist...
@@ -321,11 +338,11 @@ Processing the input file\n\
     if(Model_Number==MCE || Model_Number==MCEH){
       for(j=1;j<=ny+1;j++)
        for(i=1;i<=nx+1;i++){
-        fscanf(dp,"%Lf %Lf %Lf",&dum0,&dum1,&dum);
+        fscanf(dp,"%lf %lf %lf",&dum0,&dum1,&dum);
         u2d[i][j][1]=(real)(dum);
-        fscanf(ep,"%Lf %Lf %Lf",&dum0,&dum1,&dum);
+        fscanf(ep,"%lf %lf %lf",&dum0,&dum1,&dum);
         u2d[i][j][4]=(real)(dum*Q*u2d[i][j][1]);
-        fscanf(pp,"%Lf %Lf %Lf",&dum0,&dum1,&dum);
+        fscanf(pp,"%lf %lf %lf",&dum0,&dum1,&dum);
         PSI[i][j]=(real)(dum);
        }
     }
@@ -333,11 +350,11 @@ Processing the input file\n\
     if(Model_Number==MEPE || Model_Number==MEPEH){
       for(j=1;j<=ny+1;j++)
        for(i=1;i<=nx+1;i++){
-        fscanf(dp,"%Lf %Lf %Lf",&dum0,&dum1,&dum);
+        fscanf(dp,"%lf %lf %lf",&dum0,&dum1,&dum);
         u2d[i+2][j+2][1]=(real)(dum);
-        fscanf(ep,"%Lf %Lf %Lf",&dum0,&dum1,&dum);
+        fscanf(ep,"%lf %lf %lf",&dum0,&dum1,&dum);
         u2d[i+2][j+2][4]=(real)(dum*Q*u2d[i+2][j+2][1]);
-        fscanf(pp,"%Lf %Lf %Lf",&dum0,&dum1,&dum);
+        fscanf(pp,"%lf %lf %lf",&dum0,&dum1,&dum);
         PSI[i][j]=(real)(dum);
        }
     }
@@ -349,7 +366,7 @@ Processing the input file\n\
   }
 // choice of the GaAs impurity concentration
   else if(strcmp(s,"CIMP")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     CIMP=num;
     if(CIMP<0.){
       printf("%s: not valid impurity concentration\n",progname);
@@ -360,7 +377,7 @@ Processing the input file\n\
 
 // read the x length of the device
   else if(strcmp(s,"XLENGTH")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     LX=num;
     if(LX<=0.){
       printf("%s: not valid x-length\n",progname);
@@ -371,7 +388,7 @@ Processing the input file\n\
   }
 // read the y length of the device
   else if(strcmp(s,"YLENGTH")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     LY=num;
     if(LY<=0.){
       printf("%s: not valid y-length\n",progname);
@@ -388,35 +405,35 @@ Processing the input file\n\
       exit(EXIT_FAILURE);
     }
 // read and check the xmin value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     xmin=num;
     if(xmin<0.){
       printf("%s: not valid xmin value\n",progname);
       exit(EXIT_FAILURE);
     }
 // read and check the ymin value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     ymin=num;
     if(ymin<0.){
       printf("%s: not valid ymin value\n",progname);
       exit(EXIT_FAILURE);
     }
 // read and check the xmax value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     xmax=num;
     if(xmax>LX){
       printf("%s: not valid xmax value\n",progname);
       exit(EXIT_FAILURE);
     }
 // read and check the ymax value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     ymax=num;
     if(ymax>LY){
       printf("%s: not valid xmin value\n",progname);
       exit(EXIT_FAILURE);
     }
 // read and check the donor concentration value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     conc=num;
     if(conc<0.){
       printf("%s: not valid donor density\n",progname);
@@ -446,35 +463,35 @@ Processing the input file\n\
   else if(strcmp(s,"ACCEPTORDENSITY")==0){
     real xmin,ymin,xmax,ymax,conc;
 // read and check the xmin value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     xmin=num;
     if(xmin<0.){
       printf("%s: not valid xmin value\n",progname);
       exit(EXIT_FAILURE);
     }
 // read and check the ymin value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     ymin=num;
     if(ymin<0.){
       printf("%s: not valid ymin value\n",progname);
       exit(EXIT_FAILURE);
     }
 // read and check the xmax value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     xmax=num;
     if(xmax>LX){
       printf("%s: not valid xmax value\n",progname);
       exit(EXIT_FAILURE);
     }
 // read and check the ymax value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     ymax=num;
     if(ymax>LY){
       printf("%s: not valid xmin value\n",progname);
       exit(EXIT_FAILURE);
     }
 // read and check the acceptor concentration value
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     conc=num;
     if(conc<0.){
       printf("%s: not valid acceptor density\n",progname);
@@ -492,7 +509,7 @@ Processing the input file\n\
   }
 // read the lattice temperature
   else if(strcmp(s,"LATTICETEMPERATURE")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     TL=num;
     if(TL<=0.){
       printf("%s: not valid lattice temperature\n",progname);
@@ -502,7 +519,7 @@ Processing the input file\n\
   }
 // read the statistical weight for electrons and holes
   else if(strcmp(s,"STATISTICALWEIGHT")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     NP1=(int) num;
     printf("STATISTICAL WEIGHT = %d ---> Ok\n",NP1);
   }
@@ -552,7 +569,7 @@ Processing the input file\n\
      delt=LY/ny;
     }
 // read the "quantitative" position of the contact
-    fscanf(fp,"%Lg %Lg",&num0,&num);
+    fscanf(fp,"%lf %lf",&num0,&num);
     ipos=num0;
     fpos=num;
     if(ipos>=fpos){
@@ -571,19 +588,19 @@ Processing the input file\n\
     if(strcmp(kind,"SCHOTTKY")==0) k=1;
     if(strcmp(kind,"OHMIC")==0) k=2;
 // read the voltage applied to the contact
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     potential=num;
 // read the electron density for the reservoir at contact
 // if and only if the contact is ohmic
     if(k==2){
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       dens=num;
     }
 // read the hole density for the reservoir at contact
 // if and only if the contact is ohmic
     if(k==2 && (Model_Number==MCH || Model_Number==MEPH
                    || Model_Number==MCEH || Model_Number==MEPEH)){
-      fscanf(fp,"%Lg",&num);
+      fscanf(fp,"%lf",&num);
       denshole=num;
     }
 // internal definition of the boundary conditions
@@ -644,7 +661,7 @@ Processing the input file\n\
     printf("QUANTUM EFFECTS = YES --->Ok\n");
   }
   else if(strcmp(s,"MEDIA")==0){
-    fscanf(fp,"%Lg",&num);
+    fscanf(fp,"%lf",&num);
     if(num<0){
       printf("%s: number of media is negative\n",progname);
       exit(EXIT_FAILURE);
@@ -664,6 +681,38 @@ Processing the input file\n\
     SAVEALWAYS=1;
     printf("SAVE AT EACH TIME STEP ---> Ok\n");
   }
+  else if(strcmp(s,"FARADAY")==0){
+// Faraday equation ON or OFF
+   fscanf(fp,"%s",s);
+   if(strcmp(s,"ON")==0) FARADAYFLAG=1;
+   else if(strcmp(s,"OFF")==0) FARADAYFLAG=1;
+   else{
+     printf("%s : command FARADAY accept ON or OFF.\n",progname);
+     exit(0);
+   }
+   if(FARADAYFLAG) printf("FARADAY = ON ---> Ok\n");
+   else printf("FARADAY = OFF ---> Ok\n");
+  }
+  else if(strcmp(s,"CONSTANTMAGNETICFIELD")==0){
+// specification of a constant magnetic field in a rectangular area of the device
+    real xi,yi,xf,yf,value;
+    fscanf(fp,"%lf %lf %lf %lf %lf",&xi,&yi,&xf,&yf,&value);
+    if(xi>xf){
+      printf("%s : %f > %f in CONSTANTMAGNETICFIELD.\n",progname,xi,xf);
+      exit(0);
+    }
+    if(yi>yf){
+      printf("%s : %f > %f in CONSTANTMAGNETICFIELD.\n",progname,yi,yf);
+      exit(0);
+    }
+    for(i=1;i<=nx+1;i++)
+      for(j=1;j<=ny+1;j++)
+        if((i-0.5)*dx>=xi && (i-1.5)*dx<=xf
+         &&(j-0.5)*dy>=yi && (j-1.5)*dy<=yf){
+           B[i][j]=value;
+        }
+    printf("Constant Magnetic Field %f %f %f %f %f ---> Ok\n",xi,yi,xf,yf,value);
+  }
 // elseif(strcmp(s,"")==0){
  }while(!feof(fp));
 // computation of the maximum doping density
